// feature collection
struct Feature_Bayes
{
    // tag non/interactions
    static const int TP = 0;
    static const int TN = 1;

    // probabilities
    Vec < double > probs_t, probs_n;

    // number of features
    int nfeat;
    int nneg;
    int npos;

    // labels
    Hash < string, int > href;

    // path
    string path;

    // construct
    void construct(int nfeat, string path)
    {
        // features
        this->nfeat = nfeat;
        this->path  = path;

        // loop
        probs_t.resize(nfeat);
        probs_t.fill(1.0);
        probs_n.resize(nfeat);
        probs_n.fill(1.0);

        // update reference
        href.clear();

        // load negative and positive
        npos = add_reference(path + "/positive.txt", href, TP);
        nneg = add_reference(path + "/negative.txt", href, TN);
    }

    // make id
    string getid (string ida, string idb)
    {
        // get id
        string id = ida + "_" + idb;
        if (ida > idb)
            id = idb + "_" + ida;
        return id;
    }

    // load set
    int add_reference(string fname, Hash < string, int> &lhash, int type)
    {
        // count
        int nentry = 0;

        // load file
        Vec < string > lida, lidb;
        Format::readPairs(fname, lida, lidb);
        for (int i = 0; i < lida.size(); i++)
        {
            // get id
            string id = getid(lida[i], lidb[i]);
            if (lhash.get(id) == -1)
            {
                lhash.insert(id, type);
                nentry++;
            }
        }

        // return size
        return nentry;
    }

    // train parameters
    void train(Feature_Table& f, int n)
    {
        // reset
        double n_t = 0.0;
        double n_n = 0.0;

        // count
        for (int i = 0; i < n; i++)
        {
            for (int j = 0; j < nfeat; j++)
            {
                // get feature value
                bool evidence = f.ftable[i][j];

                // check
                if (!evidence)
                    continue;

                // get type
                int type = href.get(f.fid[i]);

                // skip entries not included in the training set
                if (type == -1)
                    continue;

                // count true positive
                if (type == TP)
                {
                    probs_t[j]++;
                    n_t++;
                } else {
                    probs_n[j]++;
                    n_n++;
                }
            }
        }

        /*/ normalize
        for (int i = 0; i < nfeat; i++)
        {
            if (n_t > 0.0)
                probs_t[i] /= n_t;
            if (n_n > 0.0)
                probs_n[i] /= n_n;
        }*/
    }

    // predict
    double predict(Vec < bool >& line)
    {
        // constant prior
        double hypo_t = 1.0;
        double hypo_n = 1.0;
        for (int i = 0; i < nfeat; i++)
        {
            // check if evidence is available
            bool evidence = line[i];
            if (!evidence)
                continue;

            // add up
            hypo_t *= probs_t[i];
            hypo_n *= probs_n[i];
        }
        
        // check
        if (hypo_t <= 0.0 || hypo_n <= 0.0)
            Msg::error("", "Hypothesis with zero probability.");

        // return
        return int (100.0 * hypo_t / hypo_n) / 100.0;
    }

    // show table
    void show(Feature_Table& f, int n)
    {
        // count
        for (int i = 0; i < n; i++)
        {
            int type = href.get(f.fid[i]);
            if (type == -1)
                continue;
            cout << f.fid[i] << " ";
            for (int j = 0; j < nfeat; j++)
                cout << f.ftable[i][j] << " ";
            cout << type << " ";
            cout << endl;
        }
    }
    
    // show probabilities
    void show_probs()
    {
        // positive
        for (int i = 0; i < nfeat; i++)
            cout << "Feature (pos.) " << i << " : " << int(probs_t[i] * 100.0) / 100.0 << endl;
        cout << endl;
        
        // negative
        for (int i = 0; i < nfeat; i++)
            cout << "Feature (neg.) " << i << " : " << int(probs_n[i] * 100.0) / 100.0 << endl;
        cout << endl;
    }
};

